#include	"h_MMC1.h"

TMMC1	MMC1;

void	MMC1_Init (void (*Sync)(void))
{
	u8 x;
	MMC1.Regs[0] = 0x0C;
	MMC1.Regs[1] = 0x00;
	MMC1.Regs[2] = 0x00;
	MMC1.Regs[3] = 0x10;
	MMC1.Latch = 0;
	MMC1.LatchPos = 0;
	MMC1.Sync = Sync;
	for (x = 0x8; x <= 0xF; x++)
		MP->SetWriteHandler(x,MMC1_Write);
	MMC1.Sync();
}

int	MMC1_SaveMI (Ar128 MI)
{
	u8 x = 0, i;
for (i = 0; i < 4; i++)	MI[x++] = MMC1.Regs[i];
			MI[x++] = MMC1.Latch;
			MI[x++] = MMC1.LatchPos;
	return x;
}

int	MMC1_LoadMI (const Ar128 MI)
{
	u8 x = 0, i;
for (i = 0; i < 4; i++)	MMC1.Regs[i]	= MI[x++];
			MMC1.Latch	= MI[x++];
			MMC1.LatchPos	= MI[x++];
	MMC1.Sync();
	return x;
}

void	MMC1_Write (int Bank, int Where, int What)
{
	u8 Reg = (Bank >> 1) & 3;
	if (What & 0x80)
	{
		MMC1.Latch = 0;
		MMC1.LatchPos = 0;
		MMC1.Regs[Reg] = ((Reg == 0) ? 0x0C : 0x00);
	}
	else
	{
		MMC1.Latch |= ((What & 1) << MMC1.LatchPos++);
		if (MMC1.LatchPos == 5)
		{
			MMC1.Regs[Reg] = MMC1.Latch & 0x1F;
			MMC1.Latch = MMC1.LatchPos = 0;
		}
	}
	MMC1.Sync();
}

void	MMC1_SyncMirror (void)
{
	switch(MMC1.Regs[0] & 3)
	{
	case 0:	MP->Mirror_S0();	break;
	case 1:	MP->Mirror_S1();	break;
	case 2:	MP->Mirror_V();		break;
	case 3:	MP->Mirror_H();		break;
	}
}

void	MMC1_SyncPRG (int Mask)
{
	int PRG0, PRG1;
	if (MMC1.Regs[0] & 0x08)
	{
		if (MMC1.Regs[0] & 0x04)
		{
			PRG0 = MMC1.Regs[3] & 0xF;
			PRG1 = 0xF;
		}
		else
		{
			PRG0 = 0x0;
			PRG1 = MMC1.Regs[3] & 0xF;
		}
		MP->SetPRG_ROM16(0x8,PRG0 & Mask);
		MP->SetPRG_ROM16(0xC,PRG1 & Mask);
	}
	else
	{
		PRG0 = MMC1.Regs[3] & 0xF;
		MP->SetPRG_ROM32(0x8,(PRG0 & Mask) >> 1);
	}
}

void	MMC1_SVROM_SyncPRG (int Mask)
{
	int PRG0, PRG1;
	if (MMC1.Regs[0] & 0x08)
	{
		if (MMC1.Regs[0] & 0x04)
		{
			PRG0 = MMC1.Regs[3] & 0xF;
			PRG1 = 0xF;
		}
		else
		{
			PRG0 = 0x0;
			PRG1 = MMC1.Regs[3] & 0xF;
		}
		PRG0 |= MMC1.Regs[1] & 0x10;
		PRG1 |= MMC1.Regs[1] & 0x10;
		MP->SetPRG_ROM16(0x8,PRG0 & Mask);
		MP->SetPRG_ROM16(0xC,PRG1 & Mask);
	}
	else
	{
		PRG0 = MMC1.Regs[3] & 0xF;
		PRG0 |= MMC1.Regs[1] & 0x10;
		MP->SetPRG_ROM32(0x8,(PRG0 & Mask) >> 1);
	}
}

void	MMC1_SyncCHR_ROM (int Mask)
{
	if (MMC1.Regs[0] & 0x10)
	{
		MP->SetCHR_ROM4(0,MMC1.Regs[1] & Mask);
		MP->SetCHR_ROM4(4,MMC1.Regs[2] & Mask);
	}
	else	MP->SetCHR_ROM8(0,(MMC1.Regs[1] & Mask) >> 1);
}

void	MMC1_SyncCHR_RAM (int Mask)
{
	if (MMC1.Regs[0] & 0x10)
	{
		MP->SetCHR_RAM4(0,MMC1.Regs[1] & Mask);
		MP->SetCHR_RAM4(4,MMC1.Regs[2] & Mask);
	}
	else	MP->SetCHR_RAM8(0,(MMC1.Regs[1] & Mask) >> 1);
}

void	MMC1_SyncWRAM (void)
{
	if (MMC1.Regs[3] & 0x10)
	{
		MP->SetPRG_OB4(0x6);
		MP->SetPRG_OB4(0x7);
	}
	else	MP->SetPRG_RAM8(0x6,0);
}

void	MMC1_SOROM_SyncWRAM (void)
{
	if (MMC1.Regs[3] & 0x10)
	{
		MP->SetPRG_OB4(0x6);
		MP->SetPRG_OB4(0x7);
	}
	else	MP->SetPRG_RAM8(0x6,((MMC1.Regs[1] & 0x10) >> 4) ^ 1); /* since the 2nd 8kb is batteried */
}