#include	"..\DLL\d_iNES.h"
#include	"..\Hardware\h_VS.h"

static	struct
{
	u8 Mirror, VROM_use;
	u8 CHR_L, CHR_H;
}	Mapper;

static	void	SyncNametables (void)
{
	u8 A,B,C,D;
	if (Mapper.VROM_use)
	{
		switch (Mapper.Mirror)
		{
		case 0:	A = C = Mapper.CHR_L;
			B = D = Mapper.CHR_H;		break;
		case 1:	A = B = Mapper.CHR_L;
			C = D = Mapper.CHR_H;		break;
		case 2:	A = B = C = D = Mapper.CHR_L;	break;
		case 3:	A = B = C = D = Mapper.CHR_H;	break;
		}
		MP->SetCHR_ROM1(0x8,A);
		MP->SetCHR_ROM1(0x9,B);
		MP->SetCHR_ROM1(0xA,C);
		MP->SetCHR_ROM1(0xB,D);
	}
	else
	{
		switch (Mapper.Mirror)
		{
		case 0:	MP->Mirror_V();		break;
		case 1:	MP->Mirror_H();		break;
		case 2:	MP->Mirror_S0();	break;
		case 3:	MP->Mirror_S1();	break;
		}
	}
}

static	void	SaveMI (Ar128 MI)
{
	u8 x = 0;
	MI[x++] = Mapper.Mirror;
	MI[x++] = Mapper.VROM_use;
	MI[x++] = Mapper.CHR_L;
	MI[x++] = Mapper.CHR_H;
}

static	void	LoadMI (const Ar128 MI)
{
	u8 x = 0;
	Mapper.Mirror		= MI[x++];
	Mapper.VROM_use	= MI[x++];
	Mapper.CHR_L		= MI[x++];
	Mapper.CHR_H		= MI[x++];
	SyncNametables();
}

static	void	PRG_Write_8 (int Bank,int Where,int What)
{
	MP->SetCHR_ROM2(0,What);
}

static	void	PRG_Write_9 (int Bank,int Where,int What)
{
	MP->SetCHR_ROM2(2,What);
}

static	void	PRG_Write_A (int Bank,int Where,int What)
{
	MP->SetCHR_ROM2(4,What);
}

static	void	PRG_Write_B (int Bank,int Where,int What)
{
	MP->SetCHR_ROM2(6,What);
}

static	void	PRG_Write_C (int Bank,int Where,int What)
{
	if (Mapper.CHR_L == (What | 0x80))
		return;
	Mapper.CHR_L = What | 0x80;
	SyncNametables();
}

static	void	PRG_Write_D (int Bank,int Where,int What)
{
	if (Mapper.CHR_H == (What | 0x80))
		return;
	Mapper.CHR_H = What | 0x80;
	SyncNametables();
}

static	void	PRG_Write_E (int Bank,int Where,int What)
{
	if ((Mapper.Mirror == (What & 0x03)) && (((What & 0x10) >> 4) == Mapper.VROM_use))
		return;
	Mapper.Mirror = What & 0x3;
	Mapper.VROM_use = (What & 0x10);
	SyncNametables();
}

static	void	PRG_Write_F (int Bank,int Where,int What)
{
	MP->SetPRG_ROM16(0x8,What);
}

static	void	HBlank (int Scanline, int Byte2001)
{
	VS_CheckCoin(Scanline);
}

static	void	UnloadMapper (void)
{
	iNES_UnloadROM();
}

static	void	InitMapper (const PMapperParam _MP, int IsHardReset)
{
	MP = _MP;

	MP->SetWriteHandler(0x8,PRG_Write_8);
	MP->SetWriteHandler(0x9,PRG_Write_9);
	MP->SetWriteHandler(0xA,PRG_Write_A);
	MP->SetWriteHandler(0xB,PRG_Write_B);
	MP->SetWriteHandler(0xC,PRG_Write_C);
	MP->SetWriteHandler(0xD,PRG_Write_D);
	MP->SetWriteHandler(0xE,PRG_Write_E);
	MP->SetWriteHandler(0xF,PRG_Write_F);

	MP->SetPRG_ROM16(0x8,0);
	MP->SetPRG_ROM16(0xC,-1);
	MP->SetCHR_ROM8(0,0);

	iNES_InitROM();
	VS_Init(IsHardReset);		/* check for VS unisystem */
}

const	TMapperInfo	MapperInfo_068 =
{
	"Sunsoft Mapper #4",
	68,
	MS_Full,
	8192,
	InitMapper,
	UnloadMapper,
	HBlank,
	NULL,
	SaveMI,
	LoadMI,
	NULL,
	VS_MenuClick
};