#include	"..\DLL\d_iNES.h"

static	struct
{
	u8 Cmd, IRQenabled;
	u16_n IRQcounter;
}	Mapper;

static	void	SaveMI (Ar128 MI)
{
	u8 x = 0;
	MI[x++] = Mapper.IRQcounter.b0;
	MI[x++] = Mapper.IRQcounter.b1;
	MI[x++] = Mapper.IRQenabled;
	MI[x++] = Mapper.Cmd;
}

static	void	LoadMI (const Ar128 MI)
{
	u8 x = 0;
	Mapper.IRQcounter.b0	= MI[x++];
	Mapper.IRQcounter.b1	= MI[x++];
	Mapper.IRQenabled	= MI[x++];
	Mapper.Cmd		= MI[x++];
}

static	void	HBlank (int Scanline, int Byte2001)
{
	if (Mapper.IRQenabled)
	{
		if (Mapper.IRQcounter.s0 <= 114)
			MP->IRQ();
		else	Mapper.IRQcounter.s0 -= 114;
	}
}

static	void	PRG_Write_89 (int Bank, int Where, int What)
{
	switch (Where)
	{
	case 0x000:	Mapper.Cmd = What & 0xF;		break;
	}
}

static	void	PRG_Write_AB (int Bank, int Where, int What)
{
	switch (Mapper.Cmd)
	{
	case 0x0:	MP->SetCHR_ROM1(0,What);	break;
	case 0x1:	MP->SetCHR_ROM1(1,What);	break;
	case 0x2:	MP->SetCHR_ROM1(2,What);	break;
	case 0x3:	MP->SetCHR_ROM1(3,What);	break;
	case 0x4:	MP->SetCHR_ROM1(4,What);	break;
	case 0x5:	MP->SetCHR_ROM1(5,What);	break;
	case 0x6:	MP->SetCHR_ROM1(6,What);	break;
	case 0x7:	MP->SetCHR_ROM1(7,What);	break;
	case 0x8:
		if (What & 0x40)
			if (What & 0x80)
				MP->SetPRG_RAM8(0x6,0);
			else;
		else MP->SetPRG_ROM8(0x6,What);		break;
	case 0x9:	MP->SetPRG_ROM8(0x8,What);	break;
	case 0xA:	MP->SetPRG_ROM8(0xA,What);	break;
	case 0xB:	MP->SetPRG_ROM8(0xC,What);	break;
	case 0xC:
		switch (What & 0x03)
		{
		case 0:	MP->Mirror_V();				break;
		case 1:	MP->Mirror_H();				break;
		case 2:	MP->Mirror_S0();			break;
		case 3:	MP->Mirror_S1();			break;
		}					break;
	case 0xD:	Mapper.IRQenabled = What;		break;
	case 0xE:	Mapper.IRQcounter.b0 = What;	break;
	case 0xF:	Mapper.IRQcounter.b1 = What;	break;
	}
}

static	void	UnloadMapper (void)
{
	iNES_UnloadROM();
}

static	void	InitMapper (const PMapperParam _MP, int IsHardReset)
{
	MP = _MP;

	MP->SetWriteHandler(0x8,PRG_Write_89);
	MP->SetWriteHandler(0x9,PRG_Write_89);
	MP->SetWriteHandler(0xA,PRG_Write_AB);
	MP->SetWriteHandler(0xB,PRG_Write_AB);

	MP->SetPRG_ROM16(0x8,0);
	MP->SetPRG_ROM16(0xC,-1);

	MP->SetCHR_ROM8(0,0);

	iNES_InitROM();

	Mapper.Cmd = Mapper.IRQenabled = 0;
	Mapper.IRQcounter.s0 = 0;
}

const	TMapperInfo	MapperInfo_069 =
{
	"Sunsoft FME-7",
	69,
	MS_Nearly,
	8192,
	InitMapper,
	UnloadMapper,
	HBlank,
	NULL,
	SaveMI,
	LoadMI,
	NULL,
	NULL
};